Class: Rley::Syntax::Production

Inherits:
Object
  • Object
show all
Defined in:
lib/rley/syntax/production.rb

Overview

In a context-free grammar, a production is a rule in which its left-hand side (LHS) consists solely of a non-terminal symbol and the right-hand side (RHS) consists of a sequence of symbols. The symbols in RHS can be either terminal or non-terminal symbols. The rule stipulates that the LHS is equivalent to the RHS, in other words every occurrence of the LHS can be substituted to corresponding RHS.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(aNonTerminal, theSymbols) ⇒ Production

Create a Production instance.



40
41
42
43
# File 'lib/rley/syntax/production.rb', line 40

def initialize(aNonTerminal, theSymbols)
  @lhs = valid_lhs(aNonTerminal)
  @rhs = valid_rhs(theSymbols)
end

Instance Attribute Details

#generative=(value) ⇒ Boolean (writeonly)

rhs members are generative (that is, they can each generate/derive a non-empty string of terminals).



26
27
28
# File 'lib/rley/syntax/production.rb', line 26

def generative=(value)
  @generative = value
end

#lhsNonTerminal (readonly) Also known as: head



17
18
19
# File 'lib/rley/syntax/production.rb', line 17

def lhs
  @lhs
end

#nameString



21
22
23
# File 'lib/rley/syntax/production.rb', line 21

def name
  @name
end

#nullable=(value) ⇒ Boolean (writeonly)



30
31
32
# File 'lib/rley/syntax/production.rb', line 30

def nullable=(value)
  @nullable = value
end

#rhsSymbolSeq (readonly) Also known as: body



14
15
16
# File 'lib/rley/syntax/production.rb', line 14

def rhs
  @rhs
end

Instance Method Details

#as(aName) ⇒ Object

A setter for the production name



78
79
80
# File 'lib/rley/syntax/production.rb', line 78

def as(aName)
  @name = aName
end

#empty?Boolean

Is the rhs empty?



47
48
49
# File 'lib/rley/syntax/production.rb', line 47

def empty?()
  return rhs.empty?
end

#generative?Boolean

Return true iff the production is generative



52
53
54
55
56
57
# File 'lib/rley/syntax/production.rb', line 52

def generative?()
  if @generative.nil?
  end

  return @generative
end

#inspectString

Returns a string containing a human-readable representation of the production.



67
68
69
70
71
72
73
74
# File 'lib/rley/syntax/production.rb', line 67

def inspect()
  result = "#<#{self.class.name}:#{object_id}"
  result << " @name=\"#{name}\""
  result << " @lhs=#{lhs.name}"
  result << " @rhs=#{rhs.inspect}"
  result << " @generative=#{@generative}>"
  return result
end

#nullable?Boolen



60
61
62
# File 'lib/rley/syntax/production.rb', line 60

def nullable?()
  return @nullable
end